#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include "common.h"
#include "macros.h"
#include "buffer.h"
#include "obj.h"

void obj_put_sym_tb_entry_be(struct buffer *buffer, struct sym_tb_entry *entry)
{
	return_if_fail(buffer != NULL);
	return_if_fail(entry != NULL);

	buffer_put_u32_be(buffer, entry->st_name);
	buffer_put_u32_be(buffer, entry->st_offset);
	buffer_put_u32_be(buffer, entry->st_size);
	buffer_put_u32_be(buffer, entry->st_visib);
	buffer_put_u32_be(buffer, entry->st_shtype);
}

void obj_put_sym_tb_entry(struct buffer *buffer, struct sym_tb_entry *entry)
{
	return_if_fail(buffer != NULL);
	return_if_fail(entry != NULL);

	buffer_put_u32(buffer, entry->st_name);
	buffer_put_u32(buffer, entry->st_offset);
	buffer_put_u32(buffer, entry->st_size);
	buffer_put_u32(buffer, entry->st_visib);
	buffer_put_u32(buffer, entry->st_shtype);
}

void obj_put_reloc_tb_entry_be(struct buffer *buffer, struct rel_tb_entry *entry)
{
	return_if_fail(buffer != NULL);
	return_if_fail(entry != NULL);

	buffer_put_u32_be(buffer, entry->r_info);
	buffer_put_u32_be(buffer, entry->r_flag);
	buffer_put_u32_be(buffer, entry->r_offset);
	buffer_put_u32_be(buffer, entry->r_shtype);
}

void obj_put_obj_header_be(struct buffer *buffer, struct obj_header *obj)
{
	return_if_fail(buffer != NULL);
	return_if_fail(obj != NULL);

	buffer_put_u8(buffer, obj->o_magic[0]);
	buffer_put_u8(buffer, obj->o_magic[1]);
	buffer_put_u8(buffer, obj->o_magic[2]);
	buffer_put_u8(buffer, obj->o_magic[3]);
	buffer_put_u32_be(buffer, obj->o_type);
	buffer_put_u16_be(buffer, obj->o_maj_vers);
	buffer_put_u16_be(buffer, obj->o_min_vers);
	buffer_put_u32_be(buffer, obj->o_src_name);
	buffer_put_u32_be(buffer, obj->o_shoff);
	buffer_put_u32_be(buffer, obj->o_shnum);
	buffer_put_u32_be(buffer, obj->o_shentsize);
	buffer_put_u32_be(buffer, obj->o_entry);
}

void obj_put_section_header_be(struct buffer *buffer, struct section_header *header)
{
	return_if_fail(buffer != NULL);
	return_if_fail(header != NULL);

	buffer_put_u32_be(buffer, header->sh_name);
	buffer_put_u32_be(buffer, header->sh_type);
	buffer_put_u32_be(buffer, header->sh_off);
	buffer_put_u32_be(buffer, header->sh_size);
	buffer_put_u32_be(buffer, header->sh_entsize);
}

void obj_get_obj_header(struct buffer *buffer, struct obj_header *obj)
{
	return_if_fail(buffer != NULL);
	return_if_fail(obj != NULL);

	obj->o_magic[0] = buffer_get_u8(buffer);
	obj->o_magic[1] = buffer_get_u8(buffer);
	obj->o_magic[2] = buffer_get_u8(buffer);
	obj->o_magic[3] = buffer_get_u8(buffer);
	obj->o_type = buffer_get_u32_be(buffer);
	obj->o_maj_vers = buffer_get_u16_be(buffer);
	obj->o_min_vers = buffer_get_u16_be(buffer);
	obj->o_src_name = buffer_get_u32_be(buffer);
	obj->o_shoff = buffer_get_u32_be(buffer);
	obj->o_shnum = buffer_get_u32_be(buffer);
	obj->o_shentsize = buffer_get_u32_be(buffer);
	obj->o_entry = buffer_get_u32_be(buffer);
}

void obj_get_section_header(struct buffer *buffer, struct section_header *header)
{
	return_if_fail(buffer != NULL);
	return_if_fail(header != NULL);

	header->sh_name = buffer_get_u32_be(buffer);
	header->sh_type = buffer_get_u32_be(buffer);
	header->sh_off = buffer_get_u32_be(buffer);
	header->sh_size = buffer_get_u32_be(buffer);
	header->sh_entsize = buffer_get_u32_be(buffer);
}

void obj_get_sym_tb_entry(struct buffer *buffer, struct sym_tb_entry *entry)
{
	return_if_fail(buffer != NULL);
	return_if_fail(entry != NULL);

	entry->st_name = buffer_get_u32_be(buffer);
	entry->st_offset = buffer_get_u32_be(buffer);
	entry->st_size = buffer_get_u32_be(buffer);
	entry->st_visib = buffer_get_u32_be(buffer);
	entry->st_shtype = buffer_get_u32_be(buffer);
}

void obj_get_reloc_tb_entry(struct buffer *buffer, struct rel_tb_entry *entry)
{
	return_if_fail(buffer != NULL);
	return_if_fail(entry != NULL);

	entry->r_info = buffer_get_u32_be(buffer);
	entry->r_flag = buffer_get_u32_be(buffer);
	entry->r_offset = buffer_get_u32_be(buffer);
	entry->r_shtype = buffer_get_u32_be(buffer);
}
